import abc
from typing import Text

from annotation_gpt.schema.result import SummarizationResult


class Summarization(object):
    @abc.abstractmethod
    def process(self, document: Text) -> SummarizationResult:
        pass



import logging
from rich.console import Console


console = Console()


class Settings(object):
    logger_name = "annotation-gpt"


settings = Settings()

logger = logging.getLogger(settings.logger_name)



from typing import TYPE_CHECKING, List, Text

if TYPE_CHECKING:
    from annotation_gpt.schema.chat import Message


class AnnotationMixin(object):
    def prompt_from_messages(self, messages: List["Message"]) -> Text:
        message_prompts: List[Text] = []
        for message in messages:
            _mp = f"<|im_start|>{message['role']}\n{message['content']}\n<|im_end|>"
            message_prompts.append(_mp)

        prompt = "\n".join(message_prompts) + "\n<|im_start|>assistant\n"
        return prompt

    def format_messages_content(
        self, messages: List["Message"], **kwargs
    ) -> List["Message"]:
        for message in messages:
            message["content"] = message["content"].format(**kwargs)
        return messages



from typing import Any, List, Optional, Text, TYPE_CHECKING, TypedDict

if TYPE_CHECKING:
    from annotation_gpt.schema.chat import Message


class CompletionChoice(TypedDict):
    finish_reason: Text
    index: int
    logprobs: Optional[Any]
    text: Text


class CompletionUsage(TypedDict):
    completion_tokens: int
    prompt_tokens: int
    total_tokens: int


class CompletionResult(TypedDict):
    choices: List[CompletionChoice]
    created: int
    id: Text
    model: Text
    object: Text
    usage: CompletionUsage


class ChatCompletionUsage(TypedDict):
    prompt_tokens: Text
    completion_tokens: Text
    total_tokens: Text


class ChatCompletionChoice(TypedDict):
    message: "Message"
    finish_reason: Text
    index: int


class ChatCompletionResult(TypedDict):
    id: Text
    object: Text
    created: int
    model: Text
    usage: ChatCompletionUsage
    choices: List[ChatCompletionChoice]



from typing import Optional, Text


def setup_openai_api_key(
    organization_id: Optional[Text] = None,
    api_key: Optional[Text] = None,
    api_base: Optional[Text] = None,
    api_type: Optional[Text] = None,
    api_version: Optional[Text] = None,
):
    """Setup the OpenAI API key.

    Parameters
    ----------
    organization_id : Optional[Text]
        The organization id.
    api_key : Optional[Text]
        The API key.
    api_base : Optional[Text]
        The API base.
    api_type : Optional[Text]
        The API type.
    api_version : Optional[Text]
        The API version.
    """

    try:
        import openai
    except ImportError:
        raise ImportError(
            "Openai is not installed. Please install it by `pip install openai`."
        )

    if organization_id:
        openai.organization = organization_id
    if api_key:
        openai.api_key = api_key
    if api_base:
        openai.api_base = api_base
    if api_type:
        openai.api_type = api_type
    if api_version:
        openai.api_version = api_version



from typing import Dict, Optional, Text, TYPE_CHECKING

from annotation_gpt.annotation.object import AnnotationMixin
from annotation_gpt.config import console, logger
from annotation_gpt.schema.chat import Message
from annotation_gpt.schema.result import SummarizationResult

try:
    import openai
except ImportError:
    console.print(
        "[yellow]Openai is not installed. Please install it by `pip install openai`.[/yellow]"
    )
    logger.warning(
        "Openai is not installed. Please install it by `pip install openai`."
    )

if TYPE_CHECKING:
    from openai.openai_object import OpenAIObject
    from annotation_gpt.schema.openai.result import (
        ChatCompletionResult,
        CompletionResult,
    )


class OpenaiSummarization(AnnotationMixin):
    model = "gpt-3.5-turbo"
    system_prompt = """\
You will summarize document pasted by user.

Instructions:
- Your summary texts should be in language {language_hint}."""
    default_language_hint = "the same as that of the document"
    default_model = "gpt-3.5-turbo"

    def __init__(
        self,
        *args,
        language_hint: Optional[Text] = None,
        model: Optional[Text] = None,
        **kwargs
    ) -> None:
        super().__init__(*args, **kwargs)

        self.language_hint = language_hint or self.default_language_hint
        self.model = model or self.default_model

    def process(
        self,
        document: Text,
        language_hint: Optional[Text] = None,
        use_chatml: bool = False,
        completion_params: Optional[Dict] = None,
    ) -> SummarizationResult:
        document = document.strip()
        language_hint = language_hint or self.language_hint
        completion_params = completion_params or {}

        messages = [
            Message(role="system", content=self.system_prompt),
            Message(role="user", content=document),
        ]
        messages = self.format_messages_content(messages, language_hint=language_hint)

        if use_chatml is True:
            completion_result: "OpenAIObject" = openai.Completion.create(
                model=self.model,
                prompt=self.prompt_from_messages(messages),
                **completion_params,
            )
            completion_result_dict: CompletionResult = (
                completion_result.to_dict_recursive()
            )
            summary = completion_result_dict["choices"][-1]["text"].strip()

        else:
            chat_completion_result: "OpenAIObject" = openai.ChatCompletion.create(
                model=self.model,
                messages=messages,
                **completion_params,
            )
            chat_completion_result_dict: ChatCompletionResult = (
                chat_completion_result.to_dict_recursive()
            )
            summary = chat_completion_result_dict["choices"][-1]["message"][
                "content"
            ].strip()

        return SummarizationResult(document=document, summary=summary)
